@@ -21,8 +21,10 @@
 
 #include <glib-object.h>
 
+/**
+ * Stock item for scan action.
+ **/
 #define	GS_STOCK_SCAN	"scan"
-#define REFDBG(o)   g_debug(G_STRLOC " %s ref count = %i", #o, G_OBJECT(o)->ref_count)
 
 G_BEGIN_DECLS
 
