@@ -68,8 +68,8 @@ GS_DEFINE_QUARK (gs_formats,		"formats");
 GS_DEFINE_QUARK (gs_scanner_front,	N_("Scan options"));
 GS_DEFINE_QUARK (gs_format,	N_("Format"));
 GS_DEFINE_QUARK (gs_sink_front,	N_("Output options"));
-GS_DEFINE_QUARK (gs_preview,		N_("Preview"));
-GS_DEFINE_QUARK (gs_hidden,		N_("Hidden"));
+GS_DEFINE_QUARK (gs_preview,		"$$PREVIEW$$");
+GS_DEFINE_QUARK (gs_hidden,		"$$HIDDEN$$");
 
 /* Common */
 
