@@ -174,7 +174,7 @@ gsppa_get_cursor_for_anchor (GtkAnchorType at)
             ct = GDK_CROSSHAIR;
         break;
         default:
-            ct = GDK_ARROW;
+            ct = GDK_LEFT_PTR;
         break;
     }
     
