@@ -72,7 +72,7 @@ gnome_scan_processor_common_init (GnomeScanProcessorCommon *object)
 									  "photos, but takes more time to process "
 									  "and can be useless for text."),
 								   GS_PARAM_GROUP_PROCESSING_COMMON,
-								   TRUE,
+								   FALSE,
 								   G_PARAM_WRITABLE);
 	gs_param_spec_set_domain (pspec, GETTEXT_PACKAGE);
 	gnome_scan_plugin_params_add (GNOME_SCAN_PLUGIN (object), pspec);
