@@ -21,7 +21,7 @@
 
 /**
  * SECTION: gnome-scan-settings
- * @short_description: Load settings accross plugins
+ * @short_description: Store settings for plugins
  *
  * When user switch from a scanner to another, he expects the settings
  * to keep almost the same. e.g. resolution should not be resetted on
@@ -30,8 +30,12 @@
  * key. This allow to apply the same #GnomeScanSettings on various
  * #GnomeScanPlugin through gnome_scan_plugin_configure().
  *
- * In the future, automatic saving to GConf or .ini would be such
- * feature easily added with this design.
+ * Since 0.5.92, #GnomeScanSettings load and store automatically
+ * settings in #GConf in /apps/gnome-scan/%prgname%/%key%. Ensure you
+ * have a call to g_set_prgname() before gnome_scan_init(). Loading is
+ * done upon construction. However, storing is done at dispose
+ * stage. Ensure to correctly ref/unref your #GnomeScanSettings
+ * instance in order to actually use this feature.
  **/
 
 #include <glib/gi18n.h>
@@ -46,7 +50,7 @@ struct _GnomeScanSettingsPrivate
 {
 	/* using GParamSpec name as key, and GValue as value */
 	GHashTable	*values;
-	GConfClient *gconf;
+	GConfClient	*gconf;
 	gboolean	disposed;
 };
 
@@ -100,7 +104,7 @@ gss_gconf_value_to_gvalue(GConfValue *value)
 			gtype = G_TYPE_INT;
 			break;
 		case GCONF_VALUE_FLOAT:
-			gtype = G_TYPE_FLOAT;
+			gtype = G_TYPE_DOUBLE;
 			break;
 		case GCONF_VALUE_BOOL:
 			gtype = G_TYPE_BOOLEAN;
@@ -119,7 +123,7 @@ gss_gconf_value_to_gvalue(GConfValue *value)
 							gconf_value_get_int(value));
 			break;
 		case GCONF_VALUE_FLOAT:
-			g_value_set_float(gvalue,
+			g_value_set_double(gvalue,
 							  gconf_value_get_float(value));
 			break;
 		case GCONF_VALUE_BOOL:
@@ -157,8 +161,15 @@ gss_gvalue_to_gconf_value(GValue *gvalue)
 		case G_TYPE_BOOLEAN:
 			type = GCONF_VALUE_BOOL;
 			break;
+		default:
+			if (g_type_is_a (gtype, G_TYPE_ENUM)) {
+				type = GCONF_VALUE_INT;
+				gtype = G_TYPE_INT;
+			}
+			else
+				return NULL;
 	}
-
+	
 	value = gconf_value_new(type);
 	
 	switch(gtype) {
@@ -167,14 +178,16 @@ gss_gvalue_to_gconf_value(GValue *gvalue)
 								   g_value_get_string(gvalue));
 			break;
 		case G_TYPE_INT:
+			g_value_init(trans, G_TYPE_INT);
+			g_value_transform (gvalue, trans);
 			gconf_value_set_int(value,
-								g_value_get_int(gvalue));
+								g_value_get_int(trans));
 			break;
 		case G_TYPE_FLOAT:
 		case G_TYPE_DOUBLE:
 			g_value_init(trans, G_TYPE_FLOAT);
 			g_value_transform (gvalue, trans);
-			gconf_value_set_int(value,
+			gconf_value_set_float(value,
 								g_value_get_float(trans));
 			break;
 		case G_TYPE_BOOLEAN:
@@ -283,9 +296,10 @@ gnome_scan_settings_class_init (GnomeScanSettingsClass *klass)
 	object_class->dispose		= gnome_scan_settings_dispose;
 	
   /**
-   * GnomeScanParamWidget::changed:
-   * @widget: The emitting widget
-   * @value: The new value
+   * GnomeScanSettings::changed:
+   * @widget:	The emitting widget
+   * @key:	The changed key
+   * @value:	The new value
    *
    * This signal is emitted when the value is changed by the user.
    **/
@@ -316,6 +330,14 @@ gnome_scan_settings_new (void)
   return GNOME_SCAN_SETTINGS (object);
 }
 
+/**
+ * gnome_scan_settings_init_value:
+ * @settings:	a #GnomeScanSettings
+ * @pspec:	a #GSParamSpec
+ *
+ * Store a new value in @settings using default value and key from
+ * @pspec.
+ **/
 void
 gnome_scan_settings_init_value (GnomeScanSettings *settings,
 								GParamSpec *pspec)
@@ -366,8 +388,8 @@ gnome_scan_settings_set (GnomeScanSettings *settings,
  **/
 void
 gnome_scan_settings_set_transform (GnomeScanSettings *settings,
-				   const gchar *name,
-				   const GValue *value)
+								   const gchar *name,
+								   const GValue *value)
 {
 	g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings) && G_IS_VALUE (value));
 	GValue *v = gnome_scan_settings_get (settings, name);
@@ -442,16 +464,16 @@ gnome_scan_settings_set_boolean (GnomeScanSettings *settings,
   g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings));
   GValue *v = g_new0 (GValue, 1);
   g_value_init (v, G_TYPE_BOOLEAN);
-  g_value_set_int (v, value);
+  g_value_set_boolean (v, value);
   gnome_scan_settings_set_transform (settings, name, v);
 }
 
 /**
  * gnome_scan_settings_set_enum:
- * @settings: a #GnomeScanSettings
- * @name: #GParamSpec name
- * @value: genum value
- * 
+ * @settings:	a #GnomeScanSettings
+ * @name:	#GParamSpec name
+ * @type:	enum GType
+ * @value:	enum value
  *
  * See: gnome_scan_settings_set_transform()
  **/
@@ -555,7 +577,7 @@ gnome_scan_settings_get (GnomeScanSettings *settings,
 }
 
 /**
- * gnome_scan_settings_get:
+ * gnome_scan_settings_get_transformed:
  * @settings: a #GnomeScanSettings
  * @name: a key
  * @type:	Transformation destination type.
@@ -657,6 +679,13 @@ gnome_scan_settings_get_int (GnomeScanSettings *settings,
   return val;
 }
 
+/**
+ * gnome_scan_settings_get_boolean:
+ * @settings:	a #GnomeScanSettings
+ * @name:	value name
+ *
+ * Returns: a gboolean 
+ **/
 gboolean
 gnome_scan_settings_get_boolean (GnomeScanSettings *settings,
                              const gchar *name)
@@ -699,6 +728,7 @@ gnome_scan_settings_get_enum (GnomeScanSettings *settings,
 /**
  * gnome_scan_settings_get_boxed:
  * @settings: a #GnomeScanSettings
+ * @type:	#GBoxed derived #GType
  * @name: a key
  * 
  * Look for a value stored in the table for @name key.
