@@ -189,7 +189,10 @@ void meta_param_destroy (MetaParam* mp)
 /* AREA */
 /* TODO : handle paper-width,paper-height */
 GSANE_DEFINE_META_PARAM(PaperSize, paper_size,
-						"tl-x,tl-y,br-x,br-y,page-format,paper-size",
+						"tl-x,tl-y,br-x,br-y,"
+						/* the following options are just swallowed */
+						"page-format,paper-size,"
+						"paper-width,paper-height,quick-format",
 						"paper-size,orig,page-orientation");
 
 static void
@@ -266,7 +269,7 @@ meta_param_paper_size_get_params			(MetaParam *mp)
 #undef set_int
 
 	/* note: decomposing area in three options orientation, origin and paper-size
-	 except these optoin to appear in this order. */
+	 expect these option to appear in this order. */
 	
 	/* ORIENTATION */
 	pspec = gs_param_spec_page_orientation ("page-orientation", N_("Page Orientation"), N_("Page orientation"),
@@ -328,15 +331,26 @@ meta_param_paper_size_get_params			(MetaParam *mp)
 
 	enumeration = g_slist_append (enumeration,
 								  gtk_paper_size_new_custom ("manual",
+		/* translator: Manual is the name of user defined paper size. */
 															 _("Manual"),
 															 w, h,
-															 GTK_UNIT_MM)); /* what if w and h are not mm, but e.g. pixel ? */
+															 GTK_UNIT_MM));
+
+	g_debug("%dx%d %s at %f", rect.width, rect.width,
+			gs_enum_get_nick_from_value(GNOME_TYPE_SCAN_UNIT,
+										gs_param_spec_get_unit(mpps->tl_x)),
+			mpps->resolution);
+
 	enumeration = g_slist_append (enumeration,
 								  gtk_paper_size_new_custom ("maximal",
 															 _("Maximal"),
-															 (gdouble) rect.width,
-															 (gdouble) rect.height,
-															 gs_param_spec_get_unit (mpps->tl_x)));
+															 gs_convert_to_mm((gdouble)rect.width,
+																			  gs_param_spec_get_unit(mpps->tl_x),
+																			  mpps->resolution),
+															 gs_convert_to_mm((gdouble)rect.height,
+																			  gs_param_spec_get_unit(mpps->tl_x),
+																			  mpps->resolution),
+															 GTK_UNIT_MM));
 	for (i = 0 ; names[i] ; i++) {
 		enumeration = g_slist_append (enumeration,
 									  gtk_paper_size_new (names[i]));
@@ -485,7 +499,7 @@ meta_param_paper_size_set_value (MetaParam *mp,
 		mpps->roi.width = w;
 		mpps->roi.height = h;
 	}
-	else if (g_str_equal (name, "origin")) {		
+	else if (g_str_equal (name, "origin")) {	
 		mpps->origin = g_value_get_pointer (value);
 		mpps->roi.x = mpps->origin->x;
 		mpps->roi.y = mpps->origin->y;
@@ -531,7 +545,10 @@ meta_param_paper_size_get_value			(MetaParam *mp, GParamSpec *spec)
 
 
 /* SOURCE */
-GSANE_DEFINE_META_PARAM(Source, source, "doc-source,source,adf,duplex", "source");
+/* TODO: handle duplex/simplex as boolean */
+GSANE_DEFINE_META_PARAM(Source, source,
+						"doc-source,source,adf,duplex",
+						"source");
 
 /* SANE 2 well known source option value */
 #define	GSANE_SOURCE_FLATBED	N_("Flatbed")
