@@ -45,7 +45,9 @@ struct _FlegitaGimpSinkPrivate
 	GimpDrawable	*drawable;
 	const GeglRectangle	*extent;
 	GeglRectangle	actual_extent;
+	gdouble			res;
 	BablFormat		*format;
+	guint			image_type;
 	GimpPixelRgn	rgn;
 	gpointer		iter;
 	GeglBuffer		*buffer;
@@ -127,6 +129,77 @@ flegita_gimp_sink_new (gint32 image_ID,
 
 
 /* INTERNALS */
+
+/* Create a new image with type computed from incomming gegl buffer
+   type. */
+static gint32
+fgs_create_image(FlegitaGimpSink *sink)
+{
+	FlegitaGimpSinkPrivate *priv = GET_PRIVATE (sink);
+	BablFormat *format = NULL;
+	guint image_type = 0;
+	guint image_base_type = 0;
+
+	g_object_get (priv->buffer, "format", &format, NULL);
+
+	switch (format->components) {
+	case 1:
+		image_type = GIMP_GRAY_IMAGE;
+		image_base_type = GIMP_GRAY;
+		break;
+	case 3:
+	default:
+		image_type = GIMP_RGB_IMAGE;
+		image_base_type = GIMP_RGB;
+		break;
+	}
+
+	sink->image_ID = gimp_image_new (1, 1, image_base_type);
+	g_debug("Image ID is %d", sink->image_ID);
+
+	gimp_image_set_resolution (sink->image_ID, priv->res, priv->res);
+	priv->image_type = image_type;
+	return sink->image_ID;
+}
+
+/* Compute babl format and update layer type according to image type
+   (not incomming gegl buffer type) */
+static BablFormat*
+fgs_compute_babl_format(FlegitaGimpSink *sink)
+{
+	FlegitaGimpSinkPrivate *priv = GET_PRIVATE (sink);
+	BablFormat *format = NULL;
+	gchar*format_name = NULL;
+	gint i;
+	gint bps;
+
+	g_object_get (priv->buffer, "format", &format, NULL);
+	bps = format->type[0]->bits;
+
+	switch (gimp_image_base_type(sink->image_ID)) {
+	case GIMP_GRAY:
+		format_name = "Y";
+		priv->image_type = GIMP_GRAY_IMAGE;
+		break;
+	default:
+		format_name = "RGB";
+		priv->image_type = GIMP_RGB_IMAGE;
+		break;
+	}
+
+	for (i = 0; i < format->components; i++) {
+		g_debug (G_STRLOC ": component %i:%s use %i bits",i,
+				 format->type[i]->instance.name,
+				 format->type[i]->bits);
+	}
+	format_name = g_strdup_printf ("%s u%i", format_name, bps > 8 ? 16 : bps);
+
+	g_debug("Format is %s",	format_name);
+
+	return (BablFormat*) babl_format (format_name);
+}
+
+
 static void
 fgs_configure (GnomeScanPlugin *plugin, GnomeScanSettings *settings)
 {
@@ -135,11 +208,7 @@ fgs_configure (GnomeScanPlugin *plugin, GnomeScanSettings *settings)
 
 	priv->layer_name = gnome_scan_settings_get_string (settings,
 													   PARAM_LAYER_NAME);
-	if (sink->is_new) {
-		gdouble res = gnome_scan_settings_get_double (settings, "resolution");
-		gimp_image_set_resolution (sink->image_ID,
-								   res, res);
-	}
+	priv->res = gnome_scan_settings_get_double (settings, "resolution");
 }
 
 static GList*
@@ -162,9 +231,8 @@ fgs_start_frame (GnomeScanPlugin *plugin)
 	FlegitaGimpSink *sink = FLEGITA_GIMP_SINK (plugin);
 	FlegitaGimpSinkPrivate *priv = GET_PRIVATE (sink);
 	GimpPixelRgn	rgn;
-	BablFormat *format = NULL;
-	gchar*format_name = NULL;
 	gint i;
+	gdouble opacity = 100.;
 
 	if (!gegl_buffer_get_pixel_count (priv->buffer)) {
 		g_warning (G_STRLOC ": No image to process !");
@@ -175,39 +243,15 @@ fgs_start_frame (GnomeScanPlugin *plugin)
 	priv->actual_extent.width = priv->extent->width - priv->extent->x;
 	priv->actual_extent.height = priv->extent->height - priv->extent->y;
 	
-	g_debug (G_STRLOC ": rect is %ix%i+%i+%i",
-			 priv->extent->width, priv->extent->height,
-			 priv->extent->x, priv->extent->y);
-	g_object_get (priv->buffer, "format", &format, NULL);
+	if (!sink->image_ID)
+		fgs_create_image(sink);
 
-	guint image_type = 0;
-	switch (format->components) {
-	case 1:
-		image_type = GIMP_GRAY_IMAGE;
-		format_name = "Y";
-		break;
-	case 3:
-	default:
-		image_type = GIMP_RGB_IMAGE;
-		format_name = "RGB";
-		break;
-	}
-
-	gint bps = format->type[0]->bits;
-	for (i = 0; i < format->components; i++) {
-		g_debug (G_STRLOC ": component %i:%s use %i bits",i,
-				 format->type[i]->instance.name,
-				 format->type[i]->bits);
-	}
-	format_name = g_strdup_printf ("%s u%i", format_name, bps > 8 ? 16 : bps);
-
-	priv->format = (BablFormat*) babl_format (format_name);
-	gdouble opacity = 100.;
+	priv->format = fgs_compute_babl_format(sink);
 
 	priv->layer = gimp_layer_new (sink->image_ID,
 								  priv->layer_name,
 								  priv->actual_extent.width, priv->actual_extent.height,
-								  image_type,
+								  priv->image_type,
 								  opacity,
 								  GIMP_NORMAL_MODE);
 
@@ -269,7 +313,7 @@ fgs_end_frame (GnomeScanPlugin *plugin)
 						  -1);
 
 	gimp_image_undo_freeze (image);
-	/* center */
+	/* by default, center new layer */
 	gimp_layer_translate (priv->layer,
 						  (gimp_image_width (image) - priv->actual_extent.width) / 2,
 						  (gimp_image_height (image) - priv->actual_extent.height) / 2);
