@@ -97,7 +97,12 @@ query (void)
 			  G_N_ELEMENTS (scan_args), G_N_ELEMENTS (scan_return),
 			  scan_args, scan_return);
 
+#if GIMP_MINOR_VERSION >= 5
+  gimp_plugin_menu_register (PROCEDURE_SCAN, "<Image>/File/Open");
+#else
   gimp_plugin_menu_register (PROCEDURE_SCAN, "<Toolbox>/File/Acquire");
+#endif
+
   gimp_plugin_icon_register (PROCEDURE_SCAN, GIMP_ICON_TYPE_IMAGE_FILE, ACTION_ICON_DIR "/scan.svg");
 
   gimp_install_procedure (PROCEDURE_SCAN_AS_LAYER,
@@ -107,7 +112,7 @@ query (void)
 			  "Étienne Bersac <bersace03@laposte.net>",
 			  "2006-2007",
 			  _("Scan as Layer..."),
-			  "RGB*",
+			  "RGB*,GRAY*",
 			  GIMP_PLUGIN,
 			  G_N_ELEMENTS (scan_as_layer_args), G_N_ELEMENTS (scan_return),
 			  scan_as_layer_args, scan_return);
@@ -137,10 +142,9 @@ run (const gchar      *name,
   gdouble xres, yres, res;
 
   /* if we are scanning, create a new image */
-  g_debug (G_STRLOC " " PROCEDURE_SCAN);
   if (g_str_equal (name, PROCEDURE_SCAN)) {
-    image_ID = gimp_image_new (1, 1, GIMP_RGB);
-    is_new = TRUE;
+	  image_ID = 0;
+	  is_new = TRUE;
   }
   /* else use the existing one. */
   else {
@@ -151,6 +155,7 @@ run (const gchar      *name,
 
   GnomeScanSettings* settings = gnome_scan_settings_new();
   GnomeScanSink *sink = flegita_gimp_sink_new(image_ID, is_new);
+  FlegitaGimpSink *fgs = FLEGITA_GIMP_SINK(sink);
   GnomeScanJob *job = gnome_scan_job_new(settings, sink);
 
   /* set default scan resolution from existing image */
@@ -162,14 +167,18 @@ run (const gchar      *name,
   }
 
   gnome_scan_job_set_sink (job, sink);
-  gnome_scan_job_set_settings(job, settings);
   GtkWidget*dialog = gnome_scan_dialog_new (NULL, job);
   gnome_scan_dialog_run (GNOME_SCAN_DIALOG (dialog));
+  g_object_unref(dialog);
+  g_object_unref(job);
+  g_object_unref(sink);
+  g_object_unref(settings);
 
-  gimp_image_get_layers (image_ID,
-			 &n_layers);
+  if (is_new && fgs->image_ID) {
+	  image_ID = fgs->image_ID;
+	  gimp_image_get_layers (image_ID,
+				 &n_layers);
 
-  if (is_new) {
     /* if we have a layer, then display */
     if (n_layers == 1) {
       gimp_display_new (image_ID);
