@@ -69,8 +69,11 @@ flegita_about(GtkButton *about,
 						  "Franklin Street, Fifth Floor Boston, MA 02110-1301, "
 						  "USA\n\n",
 						  "logo-icon-name", "flegita",
-						  /* translator : Add you in this field */
-						  "translator-credits", _("Not translated"),
+						  /* Translators: Add your names here with one name per
+						   line, like
+						     "John Smith <email1@example.com>\n
+						      Robert Foo <email2@example.com>" */
+						  "translator-credits", _("translator-credit"),
 						  "version", PACKAGE_VERSION,
 						  "website", "http://www.gnome.org/project/gnome-scan/index",
 						  "website-label", _("Gnome Scan website"),
